==> GeneticsBase_1.0.0.tar.gz: GeneticsBase/R/genotypeSummary.R <==
## library(gdata)



splitsort <- function( str, sep="/")
  {
    separate <- sapply( str, strsplit, sep, fixed=TRUE)
    sorted   <- lapply( separate, sort )
    retval   <- sapply( sorted, paste, collapse="/" )
    retval[is.na(str)] <- NA
    retval
  }
                



genotypeSummary <- function(
                            object,
                            by=NULL,
                            confidence=0.95,
                            alpha=1-confidence,
                            show=TRUE,
                            HWE.method=c("simulate","exact"),
                            simulate.p.value=TRUE,
                            B=10000,
                            verbose=FALSE,
                            includeOverall=FALSE,
                            omitRepeats=TRUE,
                            founderOnly=TRUE,
                            ...
                            )
  {

    if(founderOnly==TRUE)
    { object<-founderGeneSet(object) } 

    ## Ensure that all "Gene" names (if present) appear together in the markerInfo
    if(!is.null(markerInfo(object)$Gene))
      tGene <- markerInfo(object)$Gene
    else
      tGene <- rep(1, nrow(markerInfo(object)))

    if(!is.null(markerInfo(object)$Position))
      tPosition <- markerInfo(object)$Position
    else
      tPosition <- rep(1, nrow(markerInfo(object)))

    if(!is.null(markerInfo(object)$Name))
      tMarker <- markerInfo(object)$Name
    else
      tMarker <- rep(1, nrow(markerInfo(object)))

    ord <- order(tGene, tPosition, tMarker)
    markers <- markerInfo(object)$Name[ord]

    ## Ensure that if by is present then it matches one of the columns of sampleInfo
    if(!missing(by) && !(by %in% colnames(sampleInfo(object))))
      {
        paste("Warning! - by is not present in sampleInfo of the object",
              "Using entire dataset\n")
        by <- "ALL"
      }

    ## If the "by" info is not present in markerInfo, assign it to "ALL"
    if(missing(by))
      by <- "ALL"

    ## Saving sampleInfo as it is used in several places.
    samp.info <- sampleInfo(object)

    ##
    if(!is.null( markerInfo(object)$Gene ) )
      {
        geneList <- sort(unique(as.character( markerInfo(object)$Gene )))
        markerList <- split(markerInfo(object)$Name, markerInfo(object)$Gene)
      }
    else
      {
        geneList <- "ALL"
        markerList <- split(markerInfo(object)$Name, "ALL")
        ## markerList <- as.list(markerInfo(object)$Name)
      }

    if(!missing(by) && (by %in% colnames(sampleInfo(object))))
      {
        groupList <- levels(as.factor(sampleInfo(object)[,by]))
        if(includeOverall)
          groupList = c("ALL",groupList)
      }
    else
      groupList <- "ALL"

    nmarkers <- nmarker(object)
    nLevels <- length(groupList)
    
    ## inner calculation from markerLoop.g here
    ##    totalNoAlleles <- 0
    ##    for (marker in markers)
    ##      {
    ##        aC <- alleleCount(object, marker=marker)
    ##        count <-  apply(aC, 2, sum, na.rm=TRUE)
    ##        names(count) <- unlist(alleleLevels(object, marker))
    ##        count.na <- c(count, "NA"=sum(is.na(aC[,1])))
    ##        totalNoAlleles <- totalNoAlleles + length(count.na)
    ##      }
    ##    

    ## Compute the *most* rows we expect to see ##
    levs <- sapply(alleleLevels(object),length)^2+1
    max.rows <- (sum( levs )+1) * nLevels - 1 
    
    HWE.method <- match.arg(HWE.method)
    
    columnNames <- c("Gene", "Marker", "Position", "Group", "Genotype", 
                     "Count", "Freq", "CI-Lower", "CI-Upper",
                     "Expected", "Obs-Exp", "HWE X^2", "P-value")

    len.cols <- length(columnNames)

    resultTab <- matrix("", nrow=max.rows, ncol=len.cols)
    colnames(resultTab) <- columnNames
    

    if(by != "ALL")
      {
        factor.split <- samp.info[,by]
        subGroups <- split(samp.info, factor.split)
      }
    else
      {
        subGroups <- split(samp.info, 1)
        names(subGroups) <- "ALL"
      }
    

    counter <- 1
    for(gene in geneList)
      {
        ## add gene infomration to the table
        resultTab[counter, "Gene"] <- gene

        markers.gene <- markerList[[gene]]
        ## Marker Loop
        for(marker in markers.gene)
          {
            if(!omitRepeats)
              resultTab[counter, "Gene"] <- gene
            resultTab[counter, "Marker"] <- marker

            if(!is.null(markerInfo(object)$Position))
              resultTab[counter, "Position"] <-
                as.character(markerInfo(object)[marker,"Position"])
            else
              resultTab[counter, "Position"] <- "?"
            ## Inner most loop is starting here
            for(group in groupList)
              {
                resultTab[counter, "Group"] <- group

                ## inner calculation from markerLoop here
                ## aC <- alleleCount(object, marker=marker)

                curr.group <- group
                groupName <- group

                if(group=="ALL")
                  {
                    object.curr <- object
                  }
                else
                  {
                    subset.rows <- as.numeric(rownames(subGroups[[curr.group]]))
                    object.curr <- object[,subset.rows]
                  }


                
                
                ##    tableList <- lapply(markerNames(object), function(...) "")
                ##    names(tableList) <- markerNames(object)
                
                count.na <- table(genotypes(object,marker=marker),
                                  exclude=NULL) ## with NA's
                count <- table(genotypes(object,marker=marker))   ## without NA's
                freq  <- prop.table(count)
                n <- sum(count)

                names(count.na) <- splitsort(names(count.na))
                names(count) <- splitsort(names(count))
                names(freq) <- splitsort(names(freq))

                
                ## FIXME: These calculations are based on binomial
                ## probabilities and may not be correct for the true
                ## multinomial distributions
                stderr <- sqrt(freq*(1-freq)/n)
                ci.low  <- qbinom(p=alpha/2, prob=freq, size=n)/n
                ci.high <- qbinom(p=1-alpha/2, prob=freq, size=n)/n

                ## calculate expected genotype frequency
                markerFreq <- prop.table(table(alleles(object,marker=marker)))
                expected <- outer(markerFreq,markerFreq)*n

                upperTriangle(expected) <- upperTriangle(expected) +
                  upperTriangle(t(expected))
                lowerTriangle(expected) <- NA
                
                expectedNames <- splitsort(outer(names(markerFreq),
                                                 names(markerFreq),
                                                 paste, sep="/"))

                expected <- c(expected)
                names(expected) <- c(expectedNames)
                expected <- expected[-which(is.na(expected))]

                ## ensure we have entry for NA if needed
                if( as.character(NA) %in% names(count.na) )
                  {
                    expected <- c(expected,"NA"=0)
                    names(expected)[length(expected)] <- as.character(NA)
                  }


                ## fill count.na and count with zeros,
                ## in case some genotypes not observed
                zerofill <- function(x, list)
                  {
                    ret <- rep(0,length(list))
                    names(ret) <- list
                    ret[names(x)] <- x
                    ret
                  }
                count.na <- zerofill(count.na, names(expected))
                len.count.na <- length(count.na)
                count <- zerofill(count, names(expected))                             
                freq <- zerofill(freq, names(expected))                             
                
                ## compute obs-exp, chisquare, and p-value
                obs.exp <- count-expected
                if(HWE.method=="simulate")
                  test <- HWE.chisq(object, marker)
                else if(HWE.method=="exact")
                  test <- HWE.exact(object, marker)
                else
                  stop("Unkown HWE method: ", HWE.method)
                
                
                
                                        # Construct information matrix
                retval <- matrix("", ncol=length(columnNames), nrow=length(expected)+1)
                rownames(retval) <- c(names(expected),"")
                colnames(retval) <- columnNames

                len.expected <- length(expected)
                ##        rownames(resultTab[counter:(counter+len.expected)]) <- c(names(expected),"")
                
                
                if(!is.null(markerInfo(object)$Gene))
                  geneName <- as.character(markerInfo(object)[marker,"Gene"])
                else
                  geneName <- "?"

                if(is.na(geneName) || geneName<" ")
                  geneName <- "?"

                if(!is.null(markerInfo(object)$Position))
                  pos <- as.character(markerInfo(object)[marker,"Position"])
                else
                  pos <- "?"

                if(is.na(pos) || pos<" ")
                  pos <- "?"
                
                ## retval[1,"Gene"]     <- geneName
                ## retval[1,"Marker"]   <- marker
                ## retval[1,"Position"] <- pos

                resultTab[counter, "Gene"] <- geneName
                resultTab[counter, "Marker"] <- marker
                resultTab[counter, "Position"] <- pos

                resultTab[counter,"HWE X^2"] <- sprintf("%1.3f", test$statistic)
                resultTab[counter,"P-value"]  <- sprintf("%1.3g", test$p.value) 
                
                ## resultTab[names(expected),"Genotype"] <- sprintf("%s",names(expected))
                resultTab[counter:(counter+len.expected-1),"Genotype"] <- sprintf("%s",names(expected))
                
                count.na.loc <- which( names(expected) %in% names(count.na) )
                freq.loc <- which( names(expected) %in% names(freq) )
                ci.low.loc <- which( names(expected) %in% names(ci.low) )
                ci.high.loc <- which( names(expected) %in% names(ci.high) )
                obs.exp.loc <- which( names(expected) %in% names(obs.exp) )

                resultTab[counter + count.na.loc-1,"Count"]   <- sprintf("%6d",count.na[count.na.loc])
                resultTab[counter + freq.loc -1,"Freq"]        <- sprintf("%1.3f",freq)
                resultTab[counter + ci.low.loc -1,"CI-Lower"]  <- sprintf("%1.3f",ci.low)
                resultTab[counter + ci.high.loc-1,"CI-Upper"] <- sprintf("%1.3f",ci.high)

                resultTab[counter:(counter+len.expected-1),"Expected"] <- sprintf("%1.3f",expected)
                resultTab[counter + obs.exp.loc-1,"Obs-Exp"] <- sprintf("%1.3f",obs.exp)

                counter <- counter+len.expected+1
              }
          }
      }
    ## mask off some entries for NA
    flag <- ("NA" == resultTab[,"Genotype"])
    resultTab[flag,c("Freq","Expected","Obs-Exp")] <- ""

    ## trim off any unused rows at the end
    resultTab <- resultTab[1:(counter-1),]
    
    retval <- list()

    rownames(resultTab) <- rep("", nrow(resultTab))
    retval$data <- resultTab
    
    ##    retval$data <- do.call("rbind", tableList)
    
    ##    if(show)
    ##      print(retval, quote=F)
    ##   
    ##    invisible(retval)
    
    
    retval$footer <- paste("Confidence intervals width is 95%, computed",
                           "using the exact quantiles for the binomial",
                           "distribution. As the true distribution is ",
                           "multinomial this is only approximately correct.")
    
    retval$call <- match.call()
    class(retval) <- c("genotypeSummary","GeneticsBaseSummary")
    
    retval
  }

==> GeneticsBase_1.12.0.tar.gz: GeneticsBase/R/genotypeSummary.R <==
## library(gdata)



splitsort <- function( str, sep="/")
  {
    # sort the allele names *within* a genotype name into alphabetical order
    # this does not change the order of the genotypes, just the name
    separate <- sapply( str, strsplit, sep, fixed=TRUE)
    sorted   <- lapply( separate, sort )
    retval   <- sapply( sorted, paste, collapse="/" )
    retval[is.na(str)] <- NA
    retval
  }
                



genotypeSummary <- function(
                            object,
                            by=NULL,
                            confidence=0.95,
                            alpha=1-confidence,
                            show=TRUE,
                            HWE.method=c("simulate","exact"),
                            simulate.p.value=TRUE,
                            B=10000,
                            verbose=FALSE,
                            includeOverall=FALSE,
                            omitRepeats=TRUE,
                            founderOnly=FALSE,
                            ...
                            )
  {

    if(founderOnly==TRUE)
    { object<-founderGeneSet(object) } 

    ## Ensure that all "Gene" names (if present) appear together in the markerInfo
    if(!is.null(markerInfo(object)$Gene))
      tGene <- markerInfo(object)$Gene
    else
      tGene <- rep(1, nrow(markerInfo(object)))

    if(!is.null(markerInfo(object)$Position))
      tPosition <- markerInfo(object)$Position
    else
      tPosition <- rep(1, nrow(markerInfo(object)))

    if(!is.null(markerInfo(object)$Name))
      tMarker <- markerInfo(object)$Name
    else
      tMarker <- rep(1, nrow(markerInfo(object)))

    ord <- order(tGene, tPosition, tMarker)
    markers <- markerInfo(object)$Name[ord]

    ## Ensure that if by is present then it matches one of the columns of sampleInfo
    if(!missing(by) && !(by %in% colnames(sampleInfo(object))))
      {
        paste("Warning! - by is not present in sampleInfo of the object",
              "Using entire dataset\n")
        by <- "ALL"
      }

    ## If the "by" info is not present in markerInfo, assign it to "ALL"
    if(missing(by))
      by <- "ALL"

    ## Saving sampleInfo as it is used in several places.
    samp.info <- sampleInfo(object)

    ##
    if(!is.null( markerInfo(object)$Gene ) )
      {
        geneList <- sort(unique(as.character( markerInfo(object)$Gene )))
        markerList <- split(markerInfo(object)$Name, markerInfo(object)$Gene)
      }
    else
      {
        geneList <- "ALL"
        markerList <- split(markerInfo(object)$Name, "ALL")
        ## markerList <- as.list(markerInfo(object)$Name)
      }

    if(!missing(by) && (by %in% colnames(sampleInfo(object))))
      {
        groupList <- levels(as.factor(sampleInfo(object)[,by]))
        if(includeOverall)
          groupList = c("ALL",groupList)
      }
    else
      groupList <- "ALL"

    nMarkers <- nmarker(object)
    nLevels <- length(groupList)
    
    ## inner calculation from markerLoop.g here
    ##    totalNoAlleles <- 0
    ##    for (marker in markers)
    ##      {
    ##        aC <- alleleCount(object, marker=marker)
    ##        count <-  apply(aC, 2, sum, na.rm=TRUE)
    ##        names(count) <- unlist(alleleLevels(object, marker))
    ##        count.na <- c(count, "NA"=sum(is.na(aC[,1])))
    ##        totalNoAlleles <- totalNoAlleles + length(count.na)
    ##      }
    ##    

    ## Compute the *most* rows we expect to see ##
    levs <- sapply(alleleLevels(object),length)^2+1
    max.rows <- (sum( levs )+1) * nLevels - 1 
    
    HWE.method <- match.arg(HWE.method)
    
    columnNames <- c("Gene", "Marker", "Position", "Group", "Genotype", 
                     "Count", "Freq", "CI-Lower", "CI-Upper",
                     "Expected", "Obs-Exp", "HWE X^2", "P-value")

    len.cols <- length(columnNames)

    resultTab <- matrix("", nrow=max.rows, ncol=len.cols)
    colnames(resultTab) <- columnNames
    

    if(by != "ALL")
      {
        factor.split <- samp.info[,by]
        subGroups <- split(samp.info, factor.split)
      }
    else
      {
        subGroups <- split(samp.info, 1)
        names(subGroups) <- "ALL"
      }
    

    counter <- 1
    for(gene in geneList)
      {
        ## add gene infomration to the table
        resultTab[counter, "Gene"] <- gene

        markers.gene <- markerList[[gene]]
        ## Marker Loop
        for(marker in markers.gene)
          {
            if(!omitRepeats)
              resultTab[counter, "Gene"] <- gene
            resultTab[counter, "Marker"] <- marker

            if(!is.null(markerInfo(object)$Position))
              resultTab[counter, "Position"] <-
                as.character(markerInfo(object)[marker,"Position"])
            else
              resultTab[counter, "Position"] <- "?"
            ## Inner most loop is starting here
            for(group in groupList)
              {
                resultTab[counter, "Group"] <- group

                ## inner calculation from markerLoop here
                ## aC <- alleleCount(object, marker=marker)

                curr.group <- group
                groupName <- group

                if(group=="ALL")
                  {
                    object.curr <- object
                  }
                else
                  {
                    subset.rows <- as.numeric(rownames(subGroups[[curr.group]]))
                    object.curr <- object[,subset.rows]
                  }


                
                
                ##    tableList <- lapply(markerNames(object), function(...) "")
                ##    names(tableList) <- markerNames(object)
                
                count.na <- table(genotypes(object,marker=marker),
                                  exclude=NULL) ## with NA's
                count <- table(genotypes(object,marker=marker))   ## without NA's
                freq  <- prop.table(count)
                n <- sum(count)

                names(count.na) <- splitsort(names(count.na))
                names(count) <- splitsort(names(count))
                names(freq) <- splitsort(names(freq))

                
                ## FIXME: These calculations are based on binomial
                ## probabilities and may not be correct for the true
                ## multinomial distributions
                stderr <- sqrt(freq*(1-freq)/n)
                ci.low  <- qbinom(p=alpha/2, prob=freq, size=n)/n
                ci.high <- qbinom(p=1-alpha/2, prob=freq, size=n)/n

                ## calculate expected genotype frequency
                markerFreq <- prop.table(table(alleles(object,marker=marker)))
                expected <- outer(markerFreq,markerFreq)*n

                upperTriangle(expected) <- upperTriangle(expected) +
                  upperTriangle(t(expected))
                lowerTriangle(expected) <- NA
                
                expectedNames <- splitsort(outer(names(markerFreq),
                                                 names(markerFreq),
                                                 paste, sep="/"))

                expected <- c(expected)
                names(expected) <- c(expectedNames)
                if(sum(is.na(expected))>0)
                   expected <- expected[-which(is.na(expected))]

                ## ensure we have entry for NA if needed
                if( as.character(NA) %in% names(count.na) )
                  {
                    expected <- c(expected,"NA"=0)
                    names(expected)[length(expected)] <- as.character(NA)
                  }


                ## fill count.na and count with zeros,
                ## in case some genotypes not observed
                zerofill <- function(x, list)
                  {
                    ret <- rep(0,length(list))

                    names(ret) <- list
                    naNamesX <- which(is.na(names(x)))
                    naNamesRet <- which(is.na(names(ret)))

                    if(sum(naNamesX)>0)
                      {
                        ## hande non-NA names
                        ret[names(x)[-naNamesX]] <- x[-naNamesX]

                        ## now handle NA names
                        ret[naNamesRet] <- x[naNamesX]
                      }
                    else
                      {
                        ret[names(x)] <- x
                      }
                    ret
                  }
                count.na <- zerofill(count.na, names(expected))
                len.count.na <- length(count.na)
                count <- zerofill(count, names(expected))                             
                freq <- zerofill(freq, names(expected))                             
                
                ## compute obs-exp, chisquare, and p-value
                obs.exp <- count-expected
                if(HWE.method=="simulate")
                  test <- HWE.chisq(object, marker, founderOnly=founderOnly)
                else if(HWE.method=="exact")
                  test <- HWE.exact(object, marker, founderOnly=founderOnly)
                else
                  stop("Unkown HWE method: ", HWE.method)
                
                
                
                                        # Construct information matrix
                retval <- matrix("", ncol=length(columnNames), nrow=length(expected)+1)
                rownames(retval) <- c(names(expected),"")
                colnames(retval) <- columnNames

                len.expected <- length(expected)
                ##        rownames(resultTab[counter:(counter+len.expected)]) <- c(names(expected),"")
                
                
                if(!is.null(markerInfo(object)$Gene))
                  geneName <- as.character(markerInfo(object)[marker,"Gene"])
                else
                  geneName <- "?"

                if(is.na(geneName) || geneName<" ")
                  geneName <- "?"

                if(!is.null(markerInfo(object)$Position))
                  pos <- as.character(markerInfo(object)[marker,"Position"])
                else
                  pos <- "?"

                if(is.na(pos) || pos<" ")
                  pos <- "?"
                
                ## retval[1,"Gene"]     <- geneName
                ## retval[1,"Marker"]   <- marker
                ## retval[1,"Position"] <- pos

                resultTab[counter, "Gene"] <- geneName
                resultTab[counter, "Marker"] <- marker
                resultTab[counter, "Position"] <- pos

                resultTab[counter,"HWE X^2"] <- sprintf("%1.3f", test$statistic)
                resultTab[counter,"P-value"]  <- sprintf("%1.3g", test$p.value) 
                
                ## resultTab[names(expected),"Genotype"] <- sprintf("%s",names(expected))
                resultTab[counter:(counter+len.expected-1),"Genotype"] <- sprintf("%s",names(expected))
                
                count.na.loc <- which( names(expected) %in% names(count.na) )
                freq.loc <- which( names(expected) %in% names(freq) )
                ci.low.loc <- which( names(expected) %in% names(ci.low) )
                ci.high.loc <- which( names(expected) %in% names(ci.high) )
                obs.exp.loc <- which( names(expected) %in% names(obs.exp) )

                resultTab[counter + count.na.loc-1,"Count"]   <- sprintf("%6d",count.na[count.na.loc])
                resultTab[counter + freq.loc -1,"Freq"]        <- sprintf("%1.3f",freq)
                resultTab[counter + ci.low.loc -1,"CI-Lower"]  <- sprintf("%1.3f",ci.low)
                resultTab[counter + ci.high.loc-1,"CI-Upper"] <- sprintf("%1.3f",ci.high)

                resultTab[counter:(counter+len.expected-1),"Expected"] <- sprintf("%1.3f",expected)
                resultTab[counter + obs.exp.loc-1,"Obs-Exp"] <- sprintf("%1.3f",obs.exp)

                counter <- counter+len.expected+1
              }
          }
      }
    ## mask off some entries for NA
    flag <- ("NA" == resultTab[,"Genotype"])
    resultTab[flag,c("Freq","Expected","Obs-Exp")] <- ""

    ## trim off any unused rows at the end
    resultTab <- resultTab[1:(counter-1),, drop=FALSE]
    
    retval <- list()

    rownames(resultTab) <- rep("", nrow(resultTab))
    retval$data <- resultTab
    
    ##    retval$data <- do.call("rbind", tableList)
    
    ##    if(show)
    ##      print(retval, quote=F)
    ##   
    ##    invisible(retval)
    
    
    retval$footer <- paste("Confidence intervals width is 95%, computed",
                           "using the exact quantiles for the binomial",
                           "distribution. As the true distribution is ",
                           "multinomial this is only approximately correct.")
    
    retval$call <- match.call()
    class(retval) <- c("genotypeSummary","GeneticsBaseSummary")
    
    retval
  }
